/*
 * timePrntr
 * code by Arnold Martin RealAbsurdity()
 * originally referenced a simple word clock design from mike-gualtieri (https://www.mike-gualtieri.com/posts/arduino_word_clock)
 
  A simple clock using the Adafruit RTC, Adafruit Thermal Recepit Printer Guts and an Arduino UNO compatible MCU.

  This sketch uses the SofwareSerial Library, Adafruit Thermal Printer Library, and the Adafruit RTC library.

  Hardware Description:
  Setup for the printer is based on SofwareSerial and uses pins 9 and 10 for RX and TX respectively.
  Pin 11 is wired to a momentary contact switch and ground with internal pullup.

  The printer will print the time every thirty minutes and when the button is pushed. A long push of the
  button will also print the current date. The time is set on the RTC module according to the system time
  when the code is compiled. There is currently no method to set the time manually.
*/

// Date and time functions using a DS3231 RTC connected via I2C and Wire lib
#include "Adafruit_Thermal.h"
#include "RTClib.h"
#include "SoftwareSerial.h"
#include "logo.h"
#include "shrug.h"

RTC_DS1307 rtc;

const char* months[13] = {"\0", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
const char* dayNames[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
const char* hours[24] = {"Midnight", "One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", "Nine", "Ten", "Eleven", "Noon", "One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", "Nine", "Ten", "Eleven"};
const char* dayNums[32] = {"\0", "1st", "2nd", "3rd", "4th", "5th", "6th", "7th", "8th", "9th", "10th", "11th", "12th", "13th", "14th", "15th", "16th", "17th", "18th", "19th", "20th", "21st", "22nd", "23rd", "24th", "25th", "26th", "27th", "28th", "29th", "30th", "31st"};
const char* minutes[60] = {"\0", "One", "Two", "Three", "Four", "Five", "Six", "Seven", "Eight", "Nine",
                           "Ten", "Eleven", "Twelve", "Thirteen", "Fourteen", "Quarter", "Sixteen", "Seventeen", "Eighteen", "Nineteen",
                           "Twenty", "Twenty-one", "Twenty-two", "Twenty-three", "Twenty-four", "Twenty-five", "Twenty-six", "Twenty-seven", "Twenty-eight", "Twenty-nine",
                           "Half", "Thirty-one", "Thirty-two", "Thirty-three", "Thirty-four", "Thirty-five", "Thirty-six", "Thirty-seven", "Thirty-eight", "Thirty-nine",
                           "Twenty", "Nineteen", "Eighteen", "Seventeen", "Sixteen", "Quarter", "Fourteen", "Thirteen", "Twelve", "Eleven",
                           "Ten", "Nine", "Eight", "Seven," "Six", "Five", "Four", "Three", "Two", "One"
                          };

// Here's the new syntax when using SoftwareSerial (e.g. Arduino Uno) ----
// If using hardware serial instead, comment out or remove these lines:


#define TX_PIN 6  //10 // Arduino transmit  YELLOW WIRE  labeled RX on printer
#define RX_PIN 5  //9 // Arduino receive   WHITE WIRE   labeled TX on printer

SoftwareSerial mySerial(RX_PIN, TX_PIN); // Declare SoftwareSerial obj first
Adafruit_Thermal printer(&mySerial);     // Pass addr to printer constructor
// Then see setup() function regarding serial & printer begin() calls.

//Button and debounce variables
//constants set pin numbers, will not change
const int button1 = 16;    // the number of the pushbutton1 pin

// DEBOUNCE Variables will change:
int button1State;             // the current reading from the input pin
int lastButton1State = HIGH;   // the previous reading from the input pin

// DEBOUNCE the following variables are long's because the time, measured in miliseconds,
long lastDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50;    // the debounce time; increase if the output flickers

//comparison variables to prevent constant reprinting of the time
int curTime = 0;
int lastWriteTime = 0;

bool writeTime = false;
bool writeDate = false;

void setup () {
  pinMode(button1, INPUT_PULLUP);

  // NOTE: SOME PRINTERS NEED 9600 BAUD instead of 19200, check test page.
  mySerial.begin(9600);  // Initialize SoftwareSerial
  printer.begin();       // Initialize printer (same regardless of serial type)

  delay(3000); // wait for console opening

  if (! rtc.begin()) {
    printer.println("Couldn't find RTC");
    while (1);
  }

  if (! rtc.isrunning()) {
    printer.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
  printer.justify('C');
  printer.setSize('M');
  printer.println(F("Initializing..."));
  printer.feed(1);
  
  printer.printBitmap(logo_width, logo_height, logo);
  
  printer.justify('L');
  printer.setSize('M');
  printer.feed(1);
  printer.println(F("by Arnold Martin"));
  printer.feed(1);
  printer.justify('L');
  printer.setSize('M');
  printer.println(F("Useless?"));
  printer.println(F("Wasteful?"));
  printer.feed(1);
  
  printer.printBitmap(shrug_width, shrug_height, shrug);
  
  printer.feed(1);
  printer.println(F("ENJOY!"));
  printer.feed(3);
  delay(50);
  printTime();
  delay(500);
}

//This runs forever, checks the button for pushes, and checks the time for automated print
//on the hour and ever half hour

void loop () {
  //check for a button push
  doubleDebouncyRead();

  //set the current time from the RTC
  DateTime now = rtc.now();

  //check to see if the current time is on the hour or half hour
  if (now.minute() == 0 || now.minute() == 30) {
    //if it is check to see if the current time has been written, if so writeTime = false
    if (lastWriteTime == now.minute()) {
      writeTime = false;
    }
    //if not, write time = true
    else if (lastWriteTime != now.minute()) {
      writeTime = true;
    }
  }

  //if writeTime is true, check if current time is different than lastWritetime and print the time
  if (writeTime == true) {
    if (now.minute() != lastWriteTime) {
      lastWriteTime = now.minute();
      delay(50);
      printTime();
      delay(50);
      writeTime = false;
    }
    else {
    }
  }
  //if writeDate is true, print the date
  if (writeDate == true) {
    delay(50);
    printDate();
    delay(50);
    writeDate = false;
  }
  writeTime = false;
  writeDate = false;
}

//fuction to Print the Time in words
void printTime() {
  //wake the printer
  printer.wake();
  delay(250);

  // set the current time
  DateTime now = rtc.now();

  //always print this line
  printer.justify('L');
  printer.setSize('M');
  printer.boldOn();
  printer.println("The Current Time is...");
  delay(50);

  //If the minute value is greater or equal to 40 print "minutes to [next hour]"
  //first adjust the hour to refer to the next hour in the hours[] array
  int adjustedHour;
  if (now.minute() >= 40) {
    adjustedHour = (now.hour() + 1);
    if (adjustedHour == 24) {
      adjustedHour = 0;
    }
  }
  //Otherwise set the hour to refer to the current hour in the hours[] array
  else if (now.minute() < 40) {
    adjustedHour = now.hour();
  }

  //create convenient hour and minute strings for printing
  String hourString = hours[adjustedHour];
  String minuteString = minutes[now.minute()];

  //If the time is on the hour print this format:
  //There are two basic formats with variations built in dependant on the hour and minute values at runtime

  //if the current time is on the hour do these things
  if ((now.minute() == 0)) {
    //Based on the actual value of now.hour() do different things
    printer.justify('C');
    printer.setSize('L');
    printer.boldOn();
    switch (now.hour()) {
      //if it is midnight or noon
      case 0:
      case 12:
        printer.println(hourString);            //print the hour from the the hour[array value]
        break;                                       //and leave
      //if it is on the hour in the morning
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
      case 10:
      case 11:
        printer.println(hourString + F(" O'clock...")); //print the hour from the the hour[array value]
        printer.justify('R');
        printer.setSize('M');
        printer.println(F("...in the Morning"));             //print the phase of the day
        break;                                          //and leave
      //if it is on the hour in the afternoon
      case 13:
      case 14:
      case 15:
      case 16:
      case 17:
        printer.println(hourString + F(" O'clock")); //print the hour from the the hour[array value]
        printer.justify('R');
        printer.setSize('M');
        printer.println(F("...in the Afternoon"));           //print the phase of the day
        break;                                          //and leave
      //if it is on the hour in the evening
      case 18:
      case 19:
      case 20:
        printer.println(hourString + F(" O'clock")); //print the hour from the the hour[array value]
        printer.justify('R');
        printer.setSize('M');
        printer.println(F("...in the Evening"));             //print the phase of the day
        break;                                          //and leave
      //if it is on the hour at night
      case 21:
      case 22:
      case 23:
        printer.println(hourString + F(" O'clock")); //print the hour from the the hour[array value]
        printer.justify('R');
        printer.setSize('M');
        printer.println(F("...at Night"));                   //print the phase of the day
        break;                                          //and leave
    }
  }

  //if the time is not on the hour print do these things
  else if ((now.minute() != 0)) {
    //if the minute value is less than forty print PAST (corresponds to the minutes[] char array)
    if (now.minute() < 40) {
      if (now.minute() == 1) {
        printer.justify('C');
        printer.setSize('L');
        printer.println(minuteString);
        printer.justify('L');
        printer.setSize('M');
        printer.println(F("minute past..."));
      }
      else if (now.minute() == 15 || now.minute() == 30) {
        printer.justify('C');
        printer.setSize('L');
        printer.println(minuteString);
        printer.justify('L');
        printer.setSize('M');
        printer.println(F("past..."));
      }
      else {
        printer.justify('C');
        printer.setSize('L');
        printer.println(minuteString);
        printer.justify('L');
        printer.setSize('M');
        printer.println(F(" minutes past..."));
      }
      if (adjustedHour == 0 || adjustedHour == 12) {
        printer.justify('C');
        printer.setSize('L');
        printer.println(hourString);
      }
      else {
        printer.justify('C');
        printer.setSize('L');
        printer.println(hourString + F(" O'clock"));
      }
    }
    //if the minute value is 40 or more print TO the next hour (adjustedhour)
    else if (now.minute() >= 40) {
      if (now.minute() == 59) {
        printer.justify('C');
        printer.setSize('L');
        printer.println(minuteString);
        printer.justify('L');
        printer.setSize('M');
        printer.println(F("minute to..."));
      }
      else if (now.minute() == 45) {
        printer.justify('C');
        printer.setSize('L');
        printer.println(minuteString);
        printer.justify('L');
        printer.setSize('M');
        printer.println(F("to..."));
      }
      else {
        printer.justify('C');
        printer.setSize('L');
        printer.println(minuteString);
        printer.justify('L');
        printer.setSize('M');
        printer.println(F("minutes to..."));
      }
      if (adjustedHour == 0 || adjustedHour == 12) {
        printer.justify('C');
        printer.setSize('L');
        printer.println(hourString);
      }
      else {
        printer.justify('C');
        printer.setSize('L');
        printer.println(hourString + F(" O'clock"));
      }
    }
  }
  else {
  }
  //after all is said and done advance the printer paper and put the device to sleep
  printer.feed(3);
  printer.sleep();
}

void printDate() {
  printer.wake();
  DateTime now = rtc.now();
  String dayNameString = dayNames[now.dayOfTheWeek()];
  String monthString = months[now.month()];
  String dayNumString = dayNums[now.day()];

  printer.justify('L');
  printer.setSize('M');
  printer.println(F("Today's date is:"));
  printer.justify('C');
  printer.setSize('L');
  printer.println(dayNameString);
  printer.println(monthString + F(" ") + dayNumString);
  printer.println(now.year(), DEC);

  printer.feed(3);
  printer.sleep();
}

//a little function i wrote a while back to read a switch with debounce
void doubleDebouncyRead() {
  // read the state of the switch into a variable:
  int reading1 = digitalRead(button1);
  int reading2;

  // If the switch changed, due to noise or pressing:
  if (reading1 != lastButton1State) {
    // reset the debounce timer
    lastDebounceTime = millis();
  }
  unsigned long startTime = millis();
  unsigned long butTime = 0;
  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is it has been so for longer than the delay
    // if the button state has changed:
    if (reading1 != button1State) {
      button1State = reading1;
      //now that the button is debounced and we know noise is not setting it off, let's start a timer
      //to meeasure how long the button is pressed. A long press can have a different action than a short press
      while (button1State == LOW && digitalRead(button1) == LOW && butTime < 2500) {
        butTime = millis() - startTime;
      }
      //if the press time is less than 1 second do this
      if (button1State == LOW && butTime < 1000) {
        writeTime = true;
      }
      // if it is greater than 1 second do this
      else if (button1State == LOW && butTime > 1000) {
        writeDate = true;
      }
      else {
      }
    }
  }
  lastButton1State = reading1;
}

